;;; Copyright (c) 2004 Nikodemus Siivola <nikodemus@random-state.net>
;;;
;;; Permission is hereby granted, free of charge, to any person obtaining
;;; a copy of this software and associated documentation files (the
;;; "Software"), to deal in the Software without restriction, including
;;; without limitation the rights to use, copy, modify, merge, publish,
;;; distribute, sublicense, and/or sell copies of the Software, and to
;;; permit persons to whom the Software is furnished to do so, subject to
;;; the following conditions:
;;;
;;; The above copyright notice and this permission notice shall be
;;; included in all copies or substantial portions of the Software.
;;;
;;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
;;; NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
;;; LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
;;; OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
;;; WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(in-package :asdf-packaging-tools)

;;; Shell command wrappers

(defun copy-file (from to)
  ;; blergh
  (run-safe-shell-command "cp ~A ~A" (namestring from) (namestring to)))

(defun make-latest-symlinks (op system)
  (run-safe-shell-command "cd ~A && rm -f ~A && ln -s ~A ~A && ln -s ~A ~A"
			  ;; cd
			  (namestring (target-directory op))
			  ;; rm
			  (latest-tarball-name system)
			  ;; symlink tarball
			  (tarball-name system)
			  (latest-tarball-name system)
			  ;; symlink signature
			  (signature (tarball-name system))
			  (signature (latest-tarball-name system))))

(defun make-tarball (op system)
  (run-safe-shell-command "cd ~A && tar -czvf ~A ~A" 
		     (namestring (target-directory op))
		     (tarball-name system)
		     (enough-namestring 
		      (temporary-directory op system)
		      (target-directory op))))

(defun delete-directory (dir)
  (run-safe-shell-command "rm -r ~A" (namestring dir)))

(defun sign-tarball (op system)
  ;; We work around GPG wanting to speak directly to user (a problem
  ;; if we're running eg. in emacs) by giving it a terminal of it's own.
  (run-safe-shell-command "~A -e gpg -b -a ~A" 
			  (namestring (xterm-pathname))
			  (namestring 
			   (merge-pathnames (tarball-name system)
					    (target-directory op)))))

;;; RELEASE-OP

(defclass release-op (operation) 
  ((target-directory :initform *default-pathname-defaults*
		     :initarg :directory 
		     :accessor target-directory)))
  
(defmethod perform ((op release-op) x)
  nil)

(defmethod perform ((op release-op) (sys system))
  (let ((temp (temporary-directory op sys)))
    (ensure-directories-exist temp :verbose t)
    (oos 'release-copy-op sys :directory temp)
    (make-tarball op sys)
    (sign-tarball op sys)
    (make-latest-symlinks op sys)
    (delete-directory (temporary-directory op sys))))

;;; RELEASE-COPY-OP

(defclass release-copy-op (release-op) ())

(defmethod output-files ((op release-copy-op) (src source-file))
  (list (merge-pathnames (component-relative-pathname src)
			 (target-directory op))))

(defmethod perform ((op release-copy-op) (src source-file))
  (let ((target (ensure-directories-exist (only (output-files op src)) 
					  :verbose t)))
    (copy-file (component-pathname src) target)))

(defmethod perform ((op release-copy-op) (sys system))
  (copy-file (system-definition-pathname sys) (target-directory op)))
